@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -193,7 +194,12 @@ Turns a following A (or C-A) into a Control-Meta-A."
 (define (read-extension-key modifier)
   (if execute-extended-keys?
       (set-command-prompt-prefix!))
-  (let ((key (modifier (with-editor-interrupts-disabled keyboard-read))))
+  (let ((key
+         (modifier
+          (let ((input (with-editor-interrupts-disabled keyboard-read)))
+            (if (char? input)
+                input
+                (abort-current-command input))))))
     (if execute-extended-keys?
 	(dispatch-on-key (current-comtabs) key)
 	key)))
