@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -125,6 +126,22 @@ USA.
 #  include "error: pointers must fit in 'unsigned long'"
 #endif
 
+#ifndef OFF_T_MAX
+#  if SIZEOF_OFF_T < SIZEOF_INTMAX_T
+#    define OFF_T_MAX (~ ((~ ((intmax_t) 0)) << (CHAR_BIT * SIZEOF_OFF_T)))
+#  else
+#    define OFF_T_MAX INTMAX_MAX
+#  endif
+#endif
+
+#ifndef TIME_T_MAX
+#  if SIZEOF_TIME_T < SIZEOF_INTMAX_T
+#    define TIME_T_MAX (~ ((~ ((intmax_t) 0)) << (CHAR_BIT * SIZEOF_TIME_T)))
+#  else
+#    define TIME_T_MAX INTMAX_MAX
+#  endif
+#endif
+
 #if ((defined (__GNUC__)) && (__GNUC__ >= 3))
 #  define ATTRIBUTE(x) __attribute__ (x)
 #  define NORETURN __attribute__ ((__noreturn__))
@@ -202,7 +219,8 @@ typedef enum
   FASL_PPC32,
   FASL_X86_64,
   FASL_PPC64,
-  FASL_IA64
+  FASL_IA64,
+  FASL_ARM,
 } fasl_arch_t;
 
 /* Possible values for COMPILER_PROCESSOR_TYPE.  This identifies the
@@ -618,6 +636,11 @@ extern void win32_stack_reset (void);
 #  define MACHINE_TYPE		"ia64"
 #  define CURRENT_FASL_ARCH	FASL_IA64
 #endif
+
+#ifdef __arm__
+#  define MACHINE_TYPE		"arm"
+#  define CURRENT_FASL_ARCH	FASL_ARM
+#endif
 
 #ifdef sonyrisc
       /* On the Sony NEWS 3250, this procedure initializes the
