@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
+    Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -28,13 +29,12 @@ USA.
 #include "scheme.h"
 #include "trap.h"
 #include "lookup.h"
-#include "winder.h"
 #include "history.h"
 
 extern void * obstack_chunk_alloc (size_t);
 #define obstack_chunk_free free
 extern void preserve_signal_mask (void);
-extern void fixup_float_rounding_mode (void);
+extern void fixup_float_environment (void);
 
 /* In order to make the interpreter tail recursive (i.e.
  * to avoid calling procedures and thus saving unnecessary
@@ -253,7 +253,7 @@ abort_to_interpreter_argument (void)
 long prim_apply_error_code;
 
 void
-Interpret (void)
+Interpret (int pop_return_p)
 {
   long dispatch_code;
   struct interpreter_state_s new_state;
@@ -269,12 +269,15 @@ Interpret (void)
   bind_interpreter_state (&new_state);
   dispatch_code = (setjmp (interpreter_catch_env));
   preserve_signal_mask ();
-  fixup_float_rounding_mode ();
+  fixup_float_environment ();
 
   switch (dispatch_code)
     {
-    case 0:
-      break;
+    case 0:			/* first time */
+      if (pop_return_p)
+	goto pop_return;	/* continue */
+      else
+	break;			/* fall into eval */
 
     case PRIM_APPLY:
       PROCEED_AFTER_PRIMITIVE ();
@@ -308,6 +311,11 @@ Interpret (void)
       PROCEED_AFTER_PRIMITIVE ();
       goto pop_return;
 
+    case PRIM_RETURN_TO_C:
+      PROCEED_AFTER_PRIMITIVE ();
+      unbind_interpreter_state (interpreter_state);
+      return;
+
     case PRIM_NO_TRAP_POP_RETURN:
       PROCEED_AFTER_PRIMITIVE ();
       goto pop_return_non_trapping;
@@ -316,6 +324,11 @@ Interpret (void)
       back_out_of_primitive ();
       SIGNAL_INTERRUPT (PENDING_INTERRUPTS ());
 
+    case PRIM_ABORT_TO_C:
+      back_out_of_primitive ();
+      unbind_interpreter_state (interpreter_state);
+      return;
+
     case ERR_ARG_1_WRONG_TYPE:
       back_out_of_primitive ();
       Do_Micro_Error (ERR_ARG_1_WRONG_TYPE, true);
@@ -454,16 +467,6 @@ Interpret (void)
 	DO_NTH_THEN (RC_COMB_SAVE_VALUE, (length + 1));
       }
 
-    case TC_COMBINATION_1:
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG + 1);
-      PUSH_ENV ();
-      DO_NTH_THEN (RC_COMB_1_PROCEDURE, COMB_1_ARG_1);
-
-    case TC_COMBINATION_2:
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG + 2);
-      PUSH_ENV ();
-      DO_NTH_THEN (RC_COMB_2_FIRST_OPERAND, COMB_2_ARG_2);
-
     case TC_COMMENT:
       REDUCES_TO_NTH (COMMENT_EXPRESSION);
 
@@ -504,10 +507,6 @@ Interpret (void)
       Free += 2;
       break;
 
-    case TC_IN_PACKAGE:
-      Will_Push (CONTINUATION_SIZE);
-      PUSH_NTH_THEN (RC_EXECUTE_IN_PACKAGE_CONTINUE, IN_PACKAGE_ENVIRONMENT);
-
     case TC_LAMBDA:
     case TC_LEXPR:
       /* Deliberately omitted: EVAL_GC_CHECK (2); */
@@ -520,41 +519,17 @@ Interpret (void)
     case TC_MANIFEST_NM_VECTOR:
       EVAL_ERROR (ERR_EXECUTE_MANIFEST_VECTOR);
 
-    case TC_PCOMB0:
-      /* The argument to Will_Eventually_Push is determined by how
-	 much will be on the stack if we back out of the primitive.  */
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG);
-      Finished_Eventual_Pushing (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG);
-      SET_EXP (OBJECT_NEW_TYPE (TC_PRIMITIVE, GET_EXP));
-      goto primitive_internal_apply;
-
-    case TC_PCOMB1:
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG + 1);
-      DO_NTH_THEN (RC_PCOMB1_APPLY, PCOMB1_ARG_SLOT);
-
-    case TC_PCOMB2:
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG + 2);
-      PUSH_ENV ();
-      DO_NTH_THEN (RC_PCOMB2_DO_1, PCOMB2_ARG_2_SLOT);
-
-    case TC_PCOMB3:
-      Will_Eventually_Push (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG + 3);
-      PUSH_ENV ();
-      DO_NTH_THEN (RC_PCOMB3_DO_2, PCOMB3_ARG_3_SLOT);
-
     case TC_SCODE_QUOTE:
       SET_VAL (MEMORY_REF (GET_EXP, SCODE_QUOTE_OBJECT));
       break;
 
-    case TC_SEQUENCE_2:
+    case TC_SEQUENCE:
       Will_Push (CONTINUATION_SIZE + 1);
       PUSH_ENV ();
-      PUSH_NTH_THEN (RC_SEQ_2_DO_2, SEQUENCE_1);
+      PUSH_NTH_THEN (RC_EXECUTE_SEQUENCE_FINISH, SEQUENCE_1);
 
-    case TC_SEQUENCE_3:
-      Will_Push (CONTINUATION_SIZE + 1);
-      PUSH_ENV ();
-      PUSH_NTH_THEN (RC_SEQ_3_DO_2, SEQUENCE_1);
+    case TC_SYNTAX_ERROR:
+      EVAL_ERROR (ERR_SYNTAX_ERROR);
 
     case TC_THE_ENVIRONMENT:
       SET_VAL (GET_ENV);
@@ -563,7 +538,8 @@ Interpret (void)
     case TC_VARIABLE:
       {
 	SCHEME_OBJECT val = GET_VAL;
-	long temp = (lookup_variable (GET_ENV, GET_EXP, (&val)));
+	SCHEME_OBJECT name = (GET_VARIABLE_SYMBOL (GET_EXP));
+	long temp = (lookup_variable (GET_ENV, name, (&val)));
 	if (temp != PRIM_DONE)
 	  {
 	    /* Back out of the evaluation. */
@@ -619,27 +595,6 @@ Interpret (void)
 
   switch (OBJECT_DATUM (GET_RET))
     {
-    case RC_COMB_1_PROCEDURE:
-      POP_ENV ();
-      PUSH_VAL ();		/* Arg. 1 */
-      STACK_PUSH (SHARP_F);	/* Operator */
-      PUSH_APPLY_FRAME_HEADER (1);
-      Finished_Eventual_Pushing (CONTINUATION_SIZE);
-      DO_ANOTHER_THEN (RC_COMB_APPLY_FUNCTION, COMB_1_FN);
-
-    case RC_COMB_2_FIRST_OPERAND:
-      POP_ENV ();
-      PUSH_VAL ();
-      PUSH_ENV ();
-      DO_ANOTHER_THEN (RC_COMB_2_PROCEDURE, COMB_2_ARG_1);
-
-    case RC_COMB_2_PROCEDURE:
-      POP_ENV ();
-      PUSH_VAL ();		/* Arg 1, just calculated */
-      STACK_PUSH (SHARP_F);	/* Function */
-      PUSH_APPLY_FRAME_HEADER (2);
-      Finished_Eventual_Pushing (CONTINUATION_SIZE);
-      DO_ANOTHER_THEN (RC_COMB_APPLY_FUNCTION, COMB_2_FN);
 
     case RC_COMB_APPLY_FUNCTION:
       END_SUBPROBLEM ();
@@ -752,8 +707,6 @@ Interpret (void)
 	SCHEME_OBJECT val;
 	long code;
 
-	if (!ENVIRONMENT_P (GET_VAL))
-	  POP_RETURN_ERROR (ERR_BAD_FRAME);
 	code = (lookup_variable (GET_VAL,
 				 (MEMORY_REF (GET_EXP, ACCESS_NAME)),
 				 (&val)));
@@ -772,14 +725,18 @@ Interpret (void)
 
     case RC_EXECUTE_ASSIGNMENT_FINISH:
       {
+	SCHEME_OBJECT variable = (MEMORY_REF (GET_EXP, ASSIGN_NAME));
 	SCHEME_OBJECT old_val;
 	long code;
 
 	POP_ENV ();
-	code = (assign_variable (GET_ENV,
-				 (MEMORY_REF (GET_EXP, ASSIGN_NAME)),
-				 GET_VAL,
-				 (&old_val)));
+	if (TC_VARIABLE == (OBJECT_TYPE (variable)))
+	  code = (assign_variable (GET_ENV,
+				   (GET_VARIABLE_SYMBOL (variable)),
+				   GET_VAL,
+				   (&old_val)));
+	else
+	  code = ERR_BAD_FRAME;
 	if (code == PRIM_DONE)
 	  SET_VAL (old_val);
 	else
@@ -823,15 +780,6 @@ Interpret (void)
         POP_RETURN_ERROR (result);
       }
 
-    case RC_EXECUTE_IN_PACKAGE_CONTINUE:
-      if (ENVIRONMENT_P (GET_VAL))
-	{
-	  END_SUBPROBLEM ();
-	  SET_ENV (GET_VAL);
-	  REDUCES_TO_NTH (IN_PACKAGE_EXPRESSION);
-	}
-      POP_RETURN_ERROR (ERR_BAD_FRAME);
-
     case RC_HALT:
       Microcode_Termination (TERM_TERM_HANDLER);
 
@@ -976,10 +924,7 @@ Interpret (void)
 	    goto pop_return;
 
 	    /* After checking the number of arguments, remove the
-	       frame header since primitives do not expect it.
-
-	       NOTE: This code must match the application code which
-	       follows primitive_internal_apply.  */
+	       frame header since primitives do not expect it. */
 
 	  case TC_PRIMITIVE:
 	    if (!IMPLEMENTED_PRIMITIVE_P (Function))
@@ -987,7 +932,6 @@ Interpret (void)
 	    {
 	      unsigned long n_args = (APPLY_FRAME_N_ARGS ());
 
-
 	      /* Note that the first test below will fail for lexpr
 		 primitives.  */
 
@@ -1126,75 +1070,6 @@ Interpret (void)
 	  }
       }
 
-    case RC_MOVE_TO_ADJACENT_POINT:
-      /* GET_EXP contains the space in which we are moving */
-      {
-	long From_Count;
-	SCHEME_OBJECT Thunk;
-	SCHEME_OBJECT New_Location;
-
-	From_Count
-	  = (UNSIGNED_FIXNUM_TO_LONG (STACK_REF (TRANSLATE_FROM_DISTANCE)));
-	if (From_Count != 0)
-	  {
-	    SCHEME_OBJECT Current = STACK_REF (TRANSLATE_FROM_POINT);
-	    STACK_REF (TRANSLATE_FROM_DISTANCE)
-	      = (LONG_TO_UNSIGNED_FIXNUM (From_Count - 1));
-	    Thunk = (MEMORY_REF (Current, STATE_POINT_AFTER_THUNK));
-	    New_Location
-	      = (MEMORY_REF (Current, STATE_POINT_NEARER_POINT));
-	    (STACK_REF (TRANSLATE_FROM_POINT)) = New_Location;
-	    if ((From_Count == 1)
-		&& ((STACK_REF (TRANSLATE_TO_DISTANCE))
-		    == (LONG_TO_UNSIGNED_FIXNUM (0))))
-	      stack_pointer = (STACK_LOC (4));
-	    else
-	      SAVE_CONT ();
-	  }
-	else
-	  {
-	    long To_Count;
-	    SCHEME_OBJECT To_Location;
-	    long i;
-
-	    To_Count
-	      = ((UNSIGNED_FIXNUM_TO_LONG (STACK_REF (TRANSLATE_TO_DISTANCE)))
-		 - 1);
-	    To_Location = (STACK_REF (TRANSLATE_TO_POINT));
-	    for (i = 0; (i < To_Count); i += 1)
-	      To_Location
-		= (MEMORY_REF (To_Location, STATE_POINT_NEARER_POINT));
-	    Thunk = (MEMORY_REF (To_Location, STATE_POINT_BEFORE_THUNK));
-	    New_Location = To_Location;
-	    (STACK_REF (TRANSLATE_TO_DISTANCE))
-	      = (LONG_TO_UNSIGNED_FIXNUM (To_Count));
-	    if (To_Count == 0)
-	      stack_pointer = (STACK_LOC (4));
-	    else
-	      SAVE_CONT ();
-	  }
-	if (GET_EXP != SHARP_F)
-	  {
-	    MEMORY_SET (GET_EXP, STATE_SPACE_NEAREST_POINT, New_Location);
-	  }
-	else
-	  current_state_point = New_Location;
-	Will_Push (2);
-	STACK_PUSH (Thunk);
-	PUSH_APPLY_FRAME_HEADER (0);
-	Pushed ();
-	goto internal_apply;
-      }
-
-    case RC_INVOKE_STACK_THREAD:
-      /* Used for WITH_THREADED_STACK primitive.  */
-      Will_Push (3);
-      PUSH_VAL ();		/* Value calculated by thunk.  */
-      PUSH_EXP ();
-      PUSH_APPLY_FRAME_HEADER (1);
-      Pushed ();
-      goto internal_apply;
-
     case RC_JOIN_STACKLETS:
       unpack_control_point (GET_EXP);
       break;
@@ -1208,76 +1083,6 @@ Interpret (void)
       EXIT_CRITICAL_SECTION ({ SAVE_CONT (); });
       break;
 
-    case RC_PCOMB1_APPLY:
-      END_SUBPROBLEM ();
-      PUSH_VAL ();		/* Argument value */
-      Finished_Eventual_Pushing (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG);
-      SET_EXP (MEMORY_REF (GET_EXP, PCOMB1_FN_SLOT));
-
-    primitive_internal_apply:
-
-#ifdef COMPILE_STEPPER
-      if (trapping
-	  && (!WITHIN_CRITICAL_SECTION_P ())
-	  && ((FETCH_APPLY_TRAPPER ()) != SHARP_F))
-	{
-	  Will_Push (3);
-	  PUSH_EXP ();
-	  STACK_PUSH (FETCH_APPLY_TRAPPER ());
-	  PUSH_APPLY_FRAME_HEADER (1 + (PRIMITIVE_N_PARAMETERS (GET_EXP)));
-	  Pushed ();
-	  trapping = false;
-	  goto Apply_Non_Trapping;
-	}
-#endif /* COMPILE_STEPPER */
-
-      /* NOTE: This code must match the code in the TC_PRIMITIVE
-	 case of internal_apply.
-	 This code is simpler because:
-	 1) The arity was checked at syntax time.
-	 2) We don't have to deal with "lexpr" primitives.
-	 3) We don't need to worry about unimplemented primitives because
-	 unimplemented primitives will cause an error at invocation.  */
-      {
-	SCHEME_OBJECT primitive = GET_EXP;
-	APPLY_PRIMITIVE_FROM_INTERPRETER (primitive);
-	POP_PRIMITIVE_FRAME (PRIMITIVE_ARITY (primitive));
-	break;
-      }
-
-    case RC_PCOMB2_APPLY:
-      END_SUBPROBLEM ();
-      PUSH_VAL ();		/* Value of arg. 1 */
-      Finished_Eventual_Pushing (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG);
-      SET_EXP (MEMORY_REF (GET_EXP, PCOMB2_FN_SLOT));
-      goto primitive_internal_apply;
-
-    case RC_PCOMB2_DO_1:
-      POP_ENV ();
-      PUSH_VAL ();		/* Save value of arg. 2 */
-      DO_ANOTHER_THEN (RC_PCOMB2_APPLY, PCOMB2_ARG_1_SLOT);
-
-    case RC_PCOMB3_APPLY:
-      END_SUBPROBLEM ();
-      PUSH_VAL ();		/* Save value of arg. 1 */
-      Finished_Eventual_Pushing (CONTINUATION_SIZE + STACK_ENV_FIRST_ARG);
-      SET_EXP (MEMORY_REF (GET_EXP, PCOMB3_FN_SLOT));
-      goto primitive_internal_apply;
-
-    case RC_PCOMB3_DO_1:
-      {
-	SCHEME_OBJECT Temp = (STACK_POP ()); /* Value of arg. 3 */
-	POP_ENV ();
-	STACK_PUSH (Temp);	/* Save arg. 3 again */
-	PUSH_VAL ();		/* Save arg. 2 */
-	DO_ANOTHER_THEN (RC_PCOMB3_APPLY, PCOMB3_ARG_1_SLOT);
-      }
-
-    case RC_PCOMB3_DO_2:
-      SET_ENV (STACK_REF (0));
-      PUSH_VAL ();		/* Save value of arg. 3 */
-      DO_ANOTHER_THEN (RC_PCOMB3_DO_1, PCOMB3_ARG_2_SLOT);
-
     case RC_POP_RETURN_ERROR:
     case RC_RESTORE_VALUE:
       SET_VAL (GET_EXP);
@@ -1339,33 +1144,11 @@ Interpret (void)
       stack_pointer = (STACK_LOCATIVE_OFFSET (stack_pointer, 1));
       break;
 
-    case RC_RESTORE_TO_STATE_POINT:
-      {
-	SCHEME_OBJECT Where_To_Go = GET_EXP;
-	Will_Push (CONTINUATION_SIZE);
-	/* Restore the contents of GET_VAL after moving to point */
-	SET_EXP (GET_VAL);
-	SET_RC (RC_RESTORE_VALUE);
-	SAVE_CONT ();
-	Pushed ();
-	Translate_To_Point (Where_To_Go);
-	break;			/* We never get here.... */
-      }
-
-    case RC_SEQ_2_DO_2:
+    case RC_EXECUTE_SEQUENCE_FINISH:
       END_SUBPROBLEM ();
       POP_ENV ();
       REDUCES_TO_NTH (SEQUENCE_2);
 
-    case RC_SEQ_3_DO_2:
-      SET_ENV (STACK_REF (0));
-      DO_ANOTHER_THEN (RC_SEQ_3_DO_3, SEQUENCE_2);
-
-    case RC_SEQ_3_DO_3:
-      END_SUBPROBLEM ();
-      POP_ENV ();
-      REDUCES_TO_NTH (SEQUENCE_3);
-
     case RC_SNAP_NEED_THUNK:
       /* Don't snap thunk twice; evaluation of the thunk's body might
 	 have snapped it already.  */
